#!/usr/bin/env python

"""
Webware classes.
"""

import Generic

class Transaction(Generic.Transaction):

    """
    Webware transaction interface.
    """

    def __init__(self, trans):

        "Initialise the transaction using the Webware transaction 'trans'."

        self.trans = trans

    # Request-related methods.

    def get_request_stream(self):

        """
        A framework-specific method which returns the request stream for
        the transaction.
        """

        stream = self.trans.request().rawInput(rewind=1)
        if stream is None:
            return StringIO.StringIO("")

        return stream

    def get_request_method(self):

        """
        A framework-specific method which gets the request method.
        """

        return self.trans.request().method()

    def get_headers(self):

        """
        A framework-specific method which returns all request headers.
        """

        # NOTE: Webware doesn't really provide access to headers in the request.

        return {}

    def get_header_values(self, key):

        """
        A framework-specific method which returns a list of all request header
        values associated with the given 'key'. Note that according to RFC 2616,
        'key' is treated as a case-insensitive string.
        """

        # NOTE: Webware doesn't really provide access to headers in the request.

        return []

    def get_content_type(self):

        """
        A framework-specific method which gets the content type specified on the
        request, along with the charset employed.
        """

        return self.parse_content_type(self.trans.request().contentType())

    def get_content_charsets(self):

        """
        Returns the character set preferences.
        NOTE: Requires enhancements to HTTPRequest.
        """

        return self.trans.request().contentCharsets()

    def get_content_languages(self):

        """
        A framework-specific method which extracts language information from
        the transaction.
        NOTE: Requires enhancements to HTTPRequest.
        """

        return self.trans.request().contentLanguages()

    def get_path(self):

        """
        A framework-specific method which gets the entire path from the request.
        """

        return self.trans.request().uri()

    def get_path_info(self):

        """
        A framework-specific method which gets the "path info" (the part of the
        URL after the resource name handling the current request) from the
        request.
        """

        # NOTE: Webware does not seem to prefix the "path info" in a way
        # NOTE: consistent with the other frameworks.

        return "/" + self.trans.request().pathInfo()

    def get_query_string(self):

        """
        A framework-specific method which gets the query string from the path in
        the request.
        """

        return self.trans.request().queryString()

    # Higher level request-related methods.

    def get_fields_from_path(self):

        """
        A framework-specific method which extracts the form fields from the
        path specified in the transaction. The underlying framework may refuse
        to supply fields from the path if handling a POST transaction.

        The returned object should employ the cgi.FieldStorage interface.

        The result of this method is dependent on the WebUtils.FieldStorage
        module's FieldStorage class.
        """

        return self.trans.request().fields()

    def get_fields_from_body(self):

        """
        A framework-specific method which extracts the form fields from the
        message body in the transaction.

        The returned object should employ the cgi.FieldStorage interface.

        The result of this method is dependent on the WebUtils.FieldStorage
        module's FieldStorage class.
        """

        return self.trans.request().fields()

    def get_fields(self):

        """
        A framework-specific method which extracts the form fields from the
        transaction. Typically, the origin of the form fields will be affected
        by the method specified in the transaction.

        The returned object should employ the cgi.FieldStorage interface.
        """

        return self.trans.request().fields()

    def get_user(self):

        """
        A framework-specific method which extracts user information from the
        transaction.
        """

        # NOTE: Webware relies entirely on a CGI-style environment where the
        # NOTE: actual headers are not available. Therefore, the Web server must
        # NOTE: itself be set up to provide user support.

        try:
            return self.trans.request().remoteUser()
        except KeyError, exc:
            return None

    # Response-related methods.

    def get_response_stream(self):

        """
        A framework-specific method which returns the response stream for
        the transaction.
        """

        return self.trans.response()

    def get_response_code(self):

        """
        Get the response code associated with the transaction. If no response
        code is defined, None is returned.
        """

        # NOTE: Webware treats the response code as just another header.

        status = self.trans.response().header("Status", None)
        try:
            if status is not None:
                return int(status)
            else:
                return None
        except ValueError:
            return None

    def set_response_code(self, response_code):

        """
        Set the 'response_code' using a numeric constant defined in the HTTP
        specification.
        """

        self.trans.response().setStatus(response_code)

    def set_header_value(self, header, value):

        """
        Set the HTTP 'header' with the given 'value'.
        """

        self.trans.response().setHeader(self.format_header_value(header), self.format_header_value(value))

    def set_content_type(self, content_type):

        """
        A framework-specific method which sets the 'content_type' for the
        response.
        """

        return self.trans.response().setHeader("Content-Type", self.format_content_type(content_type))

# vim: tabstop=4 expandtab shiftwidth=4
